# -*- coding: utf-8 -*-
"""
Created on Sun Sep 26 13:45:39 2021, mod. 20.8.24

@author: philippsen
"""

# Evo Opt Rglerentwurf
from scipy.optimize import differential_evolution
import numpy as np
import control as ct
import matplotlib.pyplot as plt
plt.rcParams['pdf.fonttype'] = 42
plt.rcParams['ps.fonttype'] = 42

Kp= 1.0    #    Verstärkung Strecke
T1 = 1.0   #    Zeitkonstante
G = ct.tf(Kp,[30,31,10, 1])  # Standardregelstrecke
T = np.arange(0.0, 40.0, 0.05) # Simulationsdauer

 
# Zielfunktion
def objective(v):
    K, Tn = v
    R = ct.tf([K*Tn, K],[Tn, 0.0])    
    G0 = G*R
    Gw = ct.feedback(G0,1)
    t,y  = ct.step_response(Gw,T) # Sprungantwort
    return np.sum(t*np.abs(1-y))*0.05 # ITAE

# Parameterbereich
p_min, p_max = 0.0, 10.0
bounds = [[p_min, p_max], [p_min, p_max]]
# Durchführung der evolutionären Optimierung
result = differential_evolution(objective, bounds)
# Ergebnisse
print('Status : %s' % result['message'])
print('Anzahl Iterationen: %d' % result['nfev'])
# Überprüfung der Optimierung
solution = result['x']
evaluation = objective(solution)
print('Lösung: f(%s) = %.5f' % (solution, evaluation))
K, Tn = solution
R = ct.tf([K*Tn, K],[Tn, 0.0])    
G0 = G*R
Gw = ct.feedback(G0,1)
t, x = ct.step_response(Gw,T)
stell = ct.feedback(R, G)
t, y = ct.step_response(stell,T)

fig, ax1 = plt.subplots()
ax1.plot(t, x, "b") 
ax1.set_ylabel('x', color="blue", fontsize=14) 
plt.grid()
ax2 = ax1.twinx() 
ax2.plot(t, y, "r") 
ax2.set_ylabel('y', color="red", fontsize=14) 
ax1.set_xlabel('t [s]', fontsize=12) 
s = str("Entwurf gemäß Evo Opt Kr = %1.2f "% K)
s2 = str("Tn = %1.2f" % Tn)
plt.title(s+s2)